#
#   COMPONENT_NAME: LINUXDRV
#
#   FUNCTIONS: none
#
#   ORIGINS:
#
#
#   (C) COPYRIGHT International Business Machines Corp. 1996
#   All Rights Reserved
#   Licensed Materials - Property of IBM
#
#
#  PERL package
#
#  Package      : Support
# 
#  Description	: This package provide a hierarchy or classes which handle
#                 XML files and PAFs .
#                 This package  is  designed to be used by 'IBMnpmenu' and 
#                 'IBMprinttool' tools .
#

#################################################################################
#################################################################################
#
#                           package  Support::base
#
#################################################################################
#################################################################################
package Support::base;

use strict;

#________________________________________________________________________________
#
#  package:     Support::base
#
#  subroutine:  new
#   
#  description: the constructor of the class   
#
#  parameters:  XMLfilename  - the full name  of the XML file 
#               PAFfilename  - the full name  of the PAF file 
#
#  return:      the reference to the created object 
#
#________________________________________________________________________________
#	
sub new {

    my $class = shift;
    my $this  = {};
    
    if(  @_ ) {
	$this->{ XMLfilename } = shift;
    }
    else {
	die " Error !!! $class : XMLfilename required ";
    }

    if ( @_ ){
	$this->{ PAFfilename } = shift;
    }
    else {
	die ( "Error!!! $class : parameters : \'XMLfilename\' , " ,
             "'PAFfilename required\' " );
    }

    bless( $this , $class );
    return $this;

}

#________________________________________________________________________________
#
#  package:     Support::base
#
#  subroutine:  isValid
#   
#  description: check if the PAF file is valid    
#
#  parameters:  ( none )
#
#  return:      1   - if the PAF file is valid
#               0   - if the PAF file is not valid 
#
#________________________________________________________________________________
#
sub isValid {
    my $this = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $PAFfilename = $this->{PAFfilename};
    my @XMLstat;
    my @PAFstat;
    my $XMLtime;
    my $PAFtime;    
    # check if the file exists
    if (! -e $PAFfilename ) {
	warn "...$PAFfilename ... \n";
	return 0;
    }
    
    # get the date:time of the last modification
    # of XML and PAF
    @XMLstat=stat($XMLfilename);
    @PAFstat=stat($PAFfilename);
    $XMLtime=$XMLstat[9];
    $PAFtime=$PAFstat[9];

    # check if the PAF file is not corrupted

    open ( IN_PAF , $PAFfilename );
    my $crc_line=<IN_PAF>;
    close( IN_PAF );
    chop($crc_line);
    my $crc=substr($crc_line,3);

    if ( $PAFtime > $crc+100 ) {
  warn "...$PAFfilename ... \n";
	return 0;
    }

    # check if the date:time of PAF  is earlier than XML file


    if ( $XMLtime <= $PAFtime ) {
	return 1;
    }
    else {
  warn "...$PAFfilename ... \n";
	return 0;
    }


}


#________________________________________________________________________________
#
#  package:     Support::base
#       
#  subroutine:  eject_escape
#   
#  description: replace perl-escape sequences from the string
#
#  parameters:  the string
#
#  return:      the string without escape sequences
#
#________________________________________________________________________________
#
sub eject_escape {

    my $class = shift;
    my $string = shift;

    my $found ="";
          
    $_ = $string;
    if( m/&#[\d]{3};/ ) {
    $found = $&;
    }
    else {
  $found = "";
    }
    while ( length($found) ) {
  $_ = $found;
  m/[\d]{3}/; 
  
  my $value = chr ($&);
        $string =~  s/$found/$value/;
  $_ = $string;
        if ( m/&#[\d]{3};/ ) {
       $found = $&;
   }
  else {
      $found = "";
  }
    }
    $string;
    
} 


#################################################################################
#################################################################################
#
#                  package  Support::xmls
#
#################################################################################
#################################################################################
package Support::xmls;

use strict;
use XML::DOM;
use vars qw( @ISA );

@ISA = qw( Support::base );

#________________________________________________________________________________
#
#  package:     Support::xmls
#
#  subroutine:  new
#   
#  description:  the costructor of the  class    
#
#  parameters:  XMLfilename  - the full name  of the XML file 
#               PAFfilename  - the full name  of the PAF file 
#
#  return:      the reference to the created object 
#
#________________________________________________________________________________
#
sub new {
    my $class=shift;
    my $this=new Support::base(@_);

    my @printers=();

    $this->{printers}=\@printers;

    bless($this,$class);

    # CRITICAL CHECK ...  for existence of XML file ...
#    if(! this->checkXML) {
#	warn "Support::xmls : file ".$this->{XMLfilename}." not found \n";
#	return undef;
#    }

    if ( ! $this->isValid ) {
	$this->fillFromXML;
	$this->writeToPAF;
    }
    else {
	$this->fillFromPAF;
    }
    return $this;   
}

#________________________________________________________________________________
#
#  package:     Support::xmls
#
#  subroutine:  fillFromXML
# 
#  description: parse XML file and fill data members with coresponding values  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#
#________________________________________________________________________________
#	
sub fillFromXML {
    my $this        = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $printers      = $this->{printers};
    @$printers  = ();

    my $printersParser = new XML::DOM::Parser;
    my $printersDoc    = $printersParser->parsefile( $XMLfilename );
    my $printersList   = $printersDoc->getElementsByTagName("data");
    for(my $i=0; $i<$printersList->getLength; $i++){
	my $anonimus={};
	my $currentPrinter=$printersList->item($i);
	$anonimus->{file}  =$currentPrinter->getAttribute("file");
	my $textNode      =$currentPrinter->getLastChild; 
	$anonimus->{name}  =$textNode->toString;
	@$printers = (@$printers , $anonimus);
    } 
}

#________________________________________________________________________________
#
#  package:     Support::xmls
#
#  subroutine:  writeToPAF
#   
#  description: write data memebers to PAF file  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub writeToPAF {
    my $this        = shift;
    my $PAFfilename = $this->{PAFfilename};
    my $printers      = $this->{printers};

    open (OUT_PAF , ">".$PAFfilename);
    # the first line is the control string
    print OUT_PAF ("## ",time,"\n");
    print OUT_PAF ("##\n");
    print OUT_PAF ("## this code is generated by IBMnpmenu\n");
    print OUT_PAF ("## please don\'t edit !!! \n");
    print OUT_PAF ("##\n");
    foreach my $anonimus (@$printers) {
	print OUT_PAF ("#printer\n");
	foreach   my $key    ( keys %$anonimus )  {
	    my $value = $anonimus->{$key};
	    print OUT_PAF ("# ".$key."\n");
	    print OUT_PAF ($value."\n");
	} 
	print OUT_PAF ("#end_printer\n");
    }
    close (OUT_PAF);
}

#________________________________________________________________________________
#
#  package:     Support::xmls
#
#  subroutine:  fillFromPAF
#   
#  description: fill data members of the class form PAF 
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub fillFromPAF {
    my $this = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $PAFfilename = $this->{PAFfilename};
    my $printers = $this->{printers};
    @$printers = (); 
    my $currentPrinter = {};
    my $key = undef;
    my $value;

    my @buffer;
    my $status = "undef" ; # it can be "param" or "undef"
    open ( IN_PAF , $PAFfilename );
    @buffer=<IN_PAF>;
    close( IN_PAF );

    # start to parse the buffer ...
    chop(@buffer);

    foreach    my $item  (@buffer) {
        # out with blank lines 
	if ( $item eq '' || substr($item,0,1) eq ' ') {
	    next;
	}
        # out with comments 
	if ( substr($item,0,2) eq '##' ) {
	    next;
	}
        # start to parse printers
	if ( substr($item,0,8) eq '#printer' ) {       # start a new printer
	    $status = "param";
            # get ready for start a new printer
	    $currentPrinter = {};
	    next;
	}
	if ( substr($item,0,12) eq '#end_printer' ) {  # end the curent printer
	    $status = "undef";
            # get ready to save the priner
	    @$printers = ( @$printers , $currentPrinter);
	    next;
	}
       if (!  $key  ) {
           $key = substr($item,2);
           next;
        }

        $value=$item;
        if($status eq 'param') {
           $currentPrinter->{$key}=$value;
           $key = undef;
        }
    }  
    
}

#________________________________________________________________________________
#
#  package:     Support::xmls
#
#  subroutine:  enumPrinters
#   
#  description: returns an array with Printer names
#
#  parameters:  ( no parametters )
#
#  return:     an array with all printers   if there exists any queue
#               otherways  a blank array .
#________________________________________________________________________________
#
sub enumPrinters {
    my $this = shift;
    my $printers = $this->{printers};    
    my @result = ();
    foreach my $anonimus (@$printers) {
        @result = ( @result , $anonimus->{name} ); 
    }
    return @result;
}


#________________________________________________________________________________
#
#  package:     Support::xmls
#
#  subroutine:  getPrinterFile
#   
#  description: returns the full name of the XML file associated with a printer
#
#  parameters:  printerName - the name of the printer
#
#  return:     a scalar with the full name of the XML file associated with
#              'printerName' if there exists 'printerName' , 
#              otherways 'undef' value .
#________________________________________________________________________________
#
sub getPrinterFile {
    my $this = shift;
    my $printers = $this->{printers};    
    my $result = undef;

    if (! @_ ) {
	die ("Error  :file ",__FILE__,":line ",
             __LINE__," :function require 1 parameter .\n");
    }
    my $searchKey = shift;
    foreach my $anonimus (@$printers) {
       if ( $anonimus->{name} eq $searchKey ) {
	   $result = $anonimus->{file}; 
	   last;
       }
   }
    return $result;
}



#################################################################################
#################################################################################
#
#                  package  Support::ibmXXXXX
#
#################################################################################
#################################################################################
package Support::ibmXXXXX;

use strict;
use XML::DOM;
use vars qw( @ISA );

@ISA = qw( Support::base );

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  new
#   
#  description: the constructor of the class   
#
#  parameters:  XMLfilename  - the full name  of the XML file 
#               PAFfilename  - the full name  of the PAF file 
#
#  return:      the reference to the created object 
#
#________________________________________________________________________________
#	
sub new {
    my $class=shift;
    my $this=new Support::base(@_);
 
    my @features=();
    $this->{features}=\@features;
    bless($this,$class);

    if ( ! $this->isValid ) {
	$this->fillFromXML;
	$this->writeToPAF;
    }
    else {
	$this->fillFromPAF;
    }
    return $this;   
}

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  fillFromXML
# 
#  description: parse XML file and fill data members with coresponding values  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#
#________________________________________________________________________________
#	
sub fillFromXML {
    my $this        = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $features      = $this->{features};
    @$features  = ();

    my $featuresParser = new XML::DOM::Parser;
    my $featuresDoc    = $featuresParser->parsefile( $XMLfilename );
    my $featuresList   = $featuresDoc->getElementsByTagName("ui");

    for(my $i=0; $i<$featuresList->getLength; $i++){
	my $anonimus={};
	my $currentFeature=$featuresList->item($i);

	####  get     'id'
	$anonimus->{id}=$currentFeature->getAttribute("id");

	####  get     'translation for 'id' if exists
	my $translateList = $currentFeature->getElementsByTagName("translation");
	if( $translateList ) {
	    my $translateId=$translateList->item(0);
	    if ($translateId) {
		my $textTranslate = $translateId->getLastChild;
    $anonimus->{translation}=$this->eject_escape ( $textTranslate->toString);
	    }
	}
	
	####  get  'options'
	my @array_options=();
	$anonimus->{options}=\@array_options;
        my $options = $anonimus->{options};
	my $optionsList = $currentFeature->getElementsByTagName("option");
	for(my $k=0; $k<$optionsList->getLength; $k++ ) {
	    my $option={};
	    my $currentOption = $optionsList->item($k);

            ####  get 'option_id'
	    $option->{id}=$currentOption->getAttribute("id");

	    ####  get 'translation' for 'option_id' if exists
	    my $translateOptionList=
                       $currentOption->getElementsByTagName("translation");
	    if($translateOptionList) {
		my $translateOptionId=$translateOptionList->item(0);
		if ($translateOptionId) {
		    my $textOptionTranslate = $translateOptionId->getLastChild;
        $option->{translation}=$this->eject_escape ($textOptionTranslate->toString );
		}
		else {
		    $option->{translation} = undef;
		}
	    }
	    @$options = (@$options , $option );
	}
		
	####  get     'default'
	$anonimus->{default}=$currentFeature->getAttribute("default");

	@$features = ( @$features , $anonimus );
    }
}

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  writeToPAF
#   
#  description: write data memebers to PAF file  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub writeToPAF {
    my $this        = shift;
    my $PAFfilename = $this->{PAFfilename};
    my $features    = $this->{features};

    open (OUT_PAF , ">".$PAFfilename);
    # the first line is the control string
    print OUT_PAF ("## ",time,"\n");
    print OUT_PAF ("##\n");
    print OUT_PAF ("## this code is generated by IBMnpmenu\n");
    print OUT_PAF ("## please don\'t edit !!! \n");
    print OUT_PAF ("##\n");
    foreach my $anonimus (@$features) {
	print OUT_PAF ("#ui\n");
	foreach   my $key    ( keys %$anonimus )  {
	    my $value = $anonimus->{$key};
	    if(ref($value) ne "ARRAY") {
		print OUT_PAF ("# ".$key."\n");
		print OUT_PAF ($value."\n");
	    }
	    else {
		print OUT_PAF ("#".$key."\n");
		foreach   my $itemList   (@$value) {
		    if (ref($itemList) eq "HASH") {
			print OUT_PAF ("#option\n");
			foreach my $optionKey ( keys %$itemList ) {
			    if ( $optionKey ) {
				my $optionValue = %$itemList->{$optionKey};
				if ( defined $optionValue ) {
				    print OUT_PAF ("# ".$optionKey."\n");
				    print OUT_PAF ($optionValue."\n");
				}
			    }
			}
			print OUT_PAF ("#end_option\n");
		    }
		}
		print OUT_PAF ("#end_options\n");
	    }
	} 
	print OUT_PAF ("#end_ui\n");
    }
    close (OUT_PAF);
}


#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  fillFromPAF
#   
#  description: fill data members of the class form PAF 
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub fillFromPAF {
    my $this = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $PAFfilename = $this->{PAFfilename};
    my $features = $this->{features};
    @$features = ();

    my $currentFeature=undef;

    my $currentOptions=undef;

    my $currentOption=undef;

    my $key = undef;
    my $value;

    my @buffer;
    my $status = "undef" ; # it can be "param" , "options" , "option" or "undef"
    open ( IN_PAF , $PAFfilename );
    @buffer=<IN_PAF>;
    close( IN_PAF );

    # start to parse the buffer ...
    chop(@buffer);
    
    $status = "undef";
    foreach    my $item  (@buffer) {
        # out with blank lines 
	if ( $item eq '' || substr($item,0,1) eq ' ') {
	    next;
	}
        # out with comments 
	if ( substr($item,0,2) eq '##' ) {
	    next;
	}
        # start to parse features
	if ( substr($item,0,3) eq '#ui' ) {       # start a new feature
	    $status = "param";
            # get ready for start a new feature
	    $currentFeature = {};
	    next;
	}
	if ( substr($item,0,7) eq '#end_ui' ) {  # end the curent printerfeature
	    $status = "undef";
            # get ready to save the prinerfeature
	    @$features = ( @$features , $currentFeature );
	    next;
	}

	#### parse options
	if ( substr($item,0,8) eq '#options' ) {
	    $status = "options";
	    # get ready for starting options
	    my @list_options=();
	    $currentOptions=\@list_options;
	    $currentFeature->{options}=$currentOptions;
	    next;
	}
	if ( substr($item,0,12) eq '#end_options' ) {
	    $status = "param";
            # save the options list
	    next;
	}
	if (substr($item,0,7) eq '#option') {
	    $status = "option";
	    # get ready to get the option
	    $currentOption={};
	    next;
	}
	if (substr($item,0,11) eq '#end_option') {
	    $status = "options";
	    @$currentOptions = ( @$currentOptions , $currentOption );
	    next;
	}


	if (!  $key  ) {
	    $key = substr($item,2);
	    next;
        }

        $value=$item;
        if($status eq "param") {
	    $currentFeature->{$key}=$value;
	    $key = undef;
        }
	else {
	    $currentOption->{$key}=$value;
	    $key = undef;
	}
    }  
    
}

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  enumFeatures
#   
#  description: returns an array with Features IDs
#
#  parameters:  ( no parametters )
#
#  return:     an array with all Featues IDs   if there exists any feature
#               otherways  a blank array .
#________________________________________________________________________________
#
sub enumFeatures {
    my $this = shift;
    my $features = $this->{features};
    my @result = ();

    foreach my $anonimus (@$features) {
	@result = ( @result , $anonimus->{id} );
    }
    
    return @result;
}

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  getFeatureTranslation
#   
#  description: returns a value that contains translation of a  feature 
#               of cource if there exists any translation 
#
#  parameters:  featureID - the ID of the Feature
#
#  return:      a value that contains the translation of the 'featureID'
#               if the 'featureID' exists and it has a translation 
#               otherways the featureID
#
#________________________________________________________________________________
#
sub getFeatureTranslation {
    my $this = shift;
    my $features = $this->{features};
    my $result = undef;

    # check if the function has one parameter passed
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter .\n");
    }

    my $featureID = shift;
    
    foreach my $anonimus (@$features) {
	if( $anonimus->{id} eq $featureID ) {
	    $result = $anonimus->{translation};
	    last;
	}
    }
    if ( ! $result ) {
	$result = $featureID;
    }

    return $result;
}


#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  getFeatureDefault
#   
#  description: returns a value that contains the default option of a  feature 
#
#  parameters:  featureID - the ID of the Feature
#
#  return:      a value that contains the default option  of the 'featureID'
#               if the 'featureID' exists and it has an option 
#               otherways the 'undef' value
#
#________________________________________________________________________________
#
sub getFeatureDefault {
    my $this = shift;
    my $features = $this->{features};
    my $result = undef;

    # check if the function has one parameter passed
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter .\n");
    }

    my $featureID = shift;

    foreach my $anonimus (@$features) {
	if( $anonimus->{id} eq $featureID ) {
	    $result = $anonimus->{default};
	    last;
	}
    }
    return $result;
}

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  enumFeatureOptions
#   
#  description: returns a queue  that contains the options IDs of a feature 
#
#  parameters:  featureID - the ID of the Feature
#
#  return:      a queue that contains the options IDs of the feature 'featureID'
#               if the feature exists otherwaysan empty queue
#
#________________________________________________________________________________
#
sub enumFeatureOptions {
    my $this = shift;
    my $features = $this->{features};
    my @result = ();

    # check if the function has one parameter passed
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter .\n");
    }

    my $featureID = shift;

    foreach my $anonimus (@$features) {
	if( $anonimus->{id} eq $featureID ) {
	    my $options = $anonimus->{options};
	    foreach my $option (@$options) {
		@result = ( @result , $option->{id} );
	    };
	    last;
	}
    }
    return @result;
}

#________________________________________________________________________________
#
#  package:     Support::ibmXXXXX
#
#  subroutine:  getFeatureOptionTranslation
#   
#  description: returns a value that contains translation of a  feature  
#               option , of cource if there exists any translation 
#
#  parameters:  featureID - the ID of the Feature
#               optionID  - the ID of the Option
#
#  return:      a value that contains the translation of the 'optionID' of
#               the 'featureID' if the 'featureID' and 'optionID' exists and 
#               it has a translation otherways the 'undef' value
#
#________________________________________________________________________________
#
sub getFeatureOptionTranslation {
    my $this = shift;
    my $features = $this->{features};
    my $result = undef;

    # check if the function has one parameter passed
    if ( @_ < 2 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter .\n");
    }

    my $featureID = shift;
    my $optionID = shift;

    foreach my $anonimus (@$features) {
	if( $anonimus->{id} eq $featureID ) {
	    my $options = $anonimus->{options};
	    foreach my $option (@$options) {
		if ($option->{id} eq $optionID ) {
		    if( $option->{translation} ) {
			$result=$option->{translation};
			last;
		    }
		}
		if ( $result ) {
		    last;
		}
	    }
	}
    }
    # if it has no translation use the ID
    if (! defined  $result ) {
	$result = $optionID;
    }
 
    return $result;
}


#################################################################################
#################################################################################
#
#                  package  Support::queues
#
#################################################################################
#################################################################################
package Support::queues;

use strict;
use XML::DOM;
use vars qw( @ISA );

@ISA = qw( Support::base );


#________________________________________________________________________________
#
#  package:     Support::queues
#
#  subroutine:  new
#   
#  description: the constructor of the class   
#
#  parameters:  XMLfilename  - the full name  of the XML file 
#               PAFfilename  - the full name  of the PAF file 
#
#  return:      the reference to the created object 
#
#________________________________________________________________________________
#	
sub new {
    my $class=shift;
    my $this=new Support::base( @_ );

    my @queues=();

    $this->{queues}=\@queues;

    bless($this,$class);

    # check if XML file exists and if not create it
    $this->checkXML;


    if ( ! $this->isValid ) {
	$this->fillFromXML;
	$this->writeToPAF;
    }
    else {
	$this->fillFromPAF;
    }
    return $this;   
}


#________________________________________________________________________________
#
#  package:     Support::queues
#
#  subroutine:  checkXML
# 
#  description: check if the XML file exists , and if not create it.  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#
#________________________________________________________________________________
#		
sub checkXML {
    my $this        = shift;
    my $XMLfilename = $this->{XMLfilename};
    
    if (! -e $XMLfilename) {
	open(QCF, ">".$XMLfilename);
	print QCF "<?xml version=\"1.0\" standalone=\"yes\"?>\n";
	print QCF "<allqueues>\n";
	print QCF "</allqueues>\n";
	close QCF;
    } 
}



#________________________________________________________________________________
#
#  package:     Support::queues
#
#  subroutine:  fillFromXML
# 
#  description: parse XML file and fill data members with coresponding values  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#
#________________________________________________________________________________
#		
sub fillFromXML {
    my $this        = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $queues      = $this->{queues};
    @$queues  = ();

    my $queuesParser = new XML::DOM::Parser;
    my $queuesDoc    = $queuesParser->parsefile( $XMLfilename );
    my $queuesList   = $queuesDoc->getElementsByTagName("queue");
    for(my $i=0; $i<$queuesList->getLength; $i++){
	my $currentQueue=$queuesList->item($i);
	my $anonimus={};
	$anonimus->{queueName}=$currentQueue->getAttribute("name");
	$anonimus->{printer}  =$currentQueue->getAttribute("printer");
	$anonimus->{PPD}      =$currentQueue->getAttribute("ppd");
	$anonimus->{XML}      =$currentQueue->getAttribute("xppd");

# get transport
	my $transportList = $currentQueue->getElementsByTagName("transport");
	my $transport     = $transportList->item(0);
	$anonimus->{type}     =$transport->getAttribute("type"); 
	if( $anonimus->{type} eq "TCP/IP" ){
	    $anonimus->{address} =$transport->getAttribute("address"); 
	} elsif ( $anonimus->{type} eq "Serial" )  {
	    $anonimus->{port} =$transport->getAttribute("port"); 
	} elsif ( $anonimus->{type} eq "Parallel" ) {
	    $anonimus->{port} =$transport->getAttribute("port"); 	
	} else {
	    $anonimus->{type}="unknown";
	}

# get the type of the job : PS , PCL , RAW 
	my $languageList = $currentQueue->getElementsByTagName("language");
	my $language      = $languageList->item(0);
	$anonimus->{language} = $language->getAttribute("lang");
	if(! $anonimus->{language} ) {
	    $anonimus->{language} = "Raw"; 
	}


# get options
	my $options={};
	my $optionsList=$currentQueue->getElementsByTagName("option");
	for (my $k=0; $k<$optionsList->getLength(); $k++){
	    my $kid = $optionsList->item($k);
	    my $textNode = $kid->getLastChild;
	    my $text = $textNode->toString;
	    if( $text =~ /^([^ ]*) = ([^ ]*)/ ){
		my $key = $1;
		my $value = $2;
		$options->{$key}=$value;
	    }
	}
	$anonimus->{options}=$options;

	@$queues = ( @$queues, $anonimus );
    } 
}
#________________________________________________________________________________
#
#  package:     Support::queues
#
#  subroutine:  writeToXML
# 
#  description: write the new queues to XML file  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#
#________________________________________________________________________________
#		
sub writeToXML {
    my $this        = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $queues      = $this->{queues};
    open(QCF, ">".$XMLfilename);
    print QCF "<?xml version=\"1.0\" standalone=\"yes\"?>\n";
    print QCF "<allqueues>\n";

# rewrite the XML file .....
    
    open(QCF, ">".$XMLfilename);
    print QCF "<?xml version=\"1.0\" standalone=\"yes\"?>\n";
    print QCF "<allqueues>\n";

    foreach my $anonimus (@$queues) {
	print QCF "    <queue ";
        print QCF         "name=\"$anonimus->{queueName}\" ";
	print QCF         "printer=\"$anonimus->{printer}\" ";
	print QCF         "ppd=\"$anonimus->{PPD}\" ";
	print QCF         "xppd=\"$anonimus->{XML}\" ";
	print QCF ">\n";

	print QCF "        <transport type=\"$anonimus->{type}\" ";
	if ( $anonimus->{type} eq "TCP/IP" ) {
	    print QCF "address=\"$anonimus->{address}\" ";
	}
	else {
	    print QCF "port=\"$anonimus->{port}\" ";
	}
	print QCF "/>\n";

	# the language
	print QCF "    <language lang=\"$anonimus->{language}\" />\n";

	my $options = $anonimus->{options};
	if ( $options ) {
	    foreach my $key ( keys %$options ) {
		print QCF "        <option>";
		print QCF "$key = $options->{$key}";
		print QCF "</option>\n";
	    } 
	}
	print QCF "    </queue>\n";
    }

    print QCF "</allqueues>\n";
    

    close QCF;

}

#________________________________________________________________________________
#
#  package:     Support::queues
#
#  subroutine:  writeToPAF
#   
#  description: write data memebers to PAF file  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub writeToPAF {
    my $this        = shift;
    my $PAFfilename = $this->{PAFfilename};
    my $queues      = $this->{queues};
    
    open (OUT_PAF , ">".$PAFfilename);
    # the first line is the control string
    print OUT_PAF ("## ",time,"\n");
    print OUT_PAF ("##\n");
    print OUT_PAF ("## this code is generated by IBMnpmenu\n");
    print OUT_PAF ("## please don\'t edit !!! \n");
    print OUT_PAF ("##\n");
    foreach my $anonimus (@$queues) {
	print OUT_PAF ("#queue\n");
	foreach   my $key    ( keys %$anonimus )  {
	    my $value = $anonimus->{$key};
	    if (! $value ) { 
		next;
	    }
	    if (ref($value) ne "HASH"){
		print OUT_PAF ("# ".$key."\n");
		print OUT_PAF ($value."\n");
	    }
	    else {
		print OUT_PAF ("#".$key."\n");
		foreach   my $opt_key   ( keys %$value ) {
			print OUT_PAF ("# ".$opt_key."\n");
			my $opt_value = $value->{$opt_key};
			print OUT_PAF ($opt_value."\n");
		}
		print OUT_PAF ("#\$\n");
	    }
	} 
	print OUT_PAF ("#end_queue\n");
    }
    close (OUT_PAF);
}



#________________________________________________________________________________
#
#  package:     Support::queues
#
#  subroutine:  fillFromPAF
#   
#  description: fill data members of the class form PAF 
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub fillFromPAF {
    my $this = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $PAFfilename = $this->{PAFfilename};
    my $queues = $this->{queues};
    @$queues = (); 
    my $currentQueue = {};
    my $currentOpt   = {};
    my $key = undef;
    my $value;

    my @buffer;
    my $status = "undef" ; # it can be "param" or "options" or "undef"
    open ( IN_PAF , $PAFfilename );
    @buffer=<IN_PAF>;
    close( IN_PAF );

    # start to parse the buffer ...
    chop(@buffer);

    foreach    my $item  (@buffer) {
        # out with blank lines 
	if ( $item eq '' || substr($item,0,1) eq ' ') {
	    next;
	}
        # out with comments 
	if ( substr($item,0,2) eq '##' ) {
	    next;
	}
        # start to parse queues
	if ( substr($item,0,6) eq '#queue' ) {       # start a new queue
	    $status = "param";
            # get ready for start a new queue
	    $currentQueue  = {};
	    next;
	}
	if ( substr($item,0,10) eq '#end_queue' ) {  # end the curent queue
	    $status = "param";
            # get ready to save the queue
	    @$queues = ( @$queues , $currentQueue );
	    next;
	}
	if ( substr($item,0,8) eq '#options' ) {    # start options section
	    $status = "options";
            # get ready to start options
	    $currentOpt={};
	    next;
	}
	if ( substr($item,0,2) eq '#$' ) {          # end options section
	    $status = "param";
            # get ready to save  options 
	    $currentQueue->{options}=$currentOpt;
	    next;
	}       

       if (!  $key  ) {
           $key = substr($item,2);
           next;
        }

        $value=$item;
        if($status eq 'param') {
           $currentQueue->{$key}=$value;
           $key = undef;
        }
        elsif ($status eq 'options' ) {
          $currentOpt->{$key}=$value;
           $key = undef;
        }
 
    }  
    
}


#________________________________________________________________________________
#
#  subroutine:  enumQueues
#   
#  description: returns an array with the Queue names
#
#  parameters:  ( no parametters )
#
#  return:     an array with all queues   if there exists any queue
#               otherways  a blank array .
#________________________________________________________________________________
#
sub enumQueues {
    my $this = shift;
    my $queues = $this->{queues};
    my @result = ();

    foreach my $anonimus (@$queues) {
	@result = ( @result , $anonimus->{queueName} );
    }
    return @result;
}

#________________________________________________________________________________
#
#  subroutine:  deleteQueue
#   
#  description: deletes a queue from queues list 
#
#  parameters:  queueName - the name of the queue which will be deleted
#
#  return:     ( no value )
#
#________________________________________________________________________________
#
sub deleteQueue {
    my $this = shift;
    my $queues = $this->{queues};
    my @new_queues = ();

    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }

    my $queueName = shift;

    foreach my $anonimus (@$queues) {
	if (  $anonimus and $anonimus->{queueName} ne $queueName ) {
	    @new_queues =( @new_queues , $anonimus );
	}
    }
    $this->{queues} = \@new_queues;
}
#________________________________________________________________________________
#
#  subroutine:  changeQueueName
#   
#  description: change the queuename 
#
#  parameters:  queueName - the old queueName
#               newQueueName - the new queueName
#
#  return:     ( no value )
#
#________________________________________________________________________________
#
sub changeQueueName {
    my $this = shift;
    my $queues = $this->{queues};

    if ( @_ < 2 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter(s) .\n");
    }

    my $queueName    = shift;
    my $newQueueName = shift;

    foreach my $anonimus (@$queues) {
	if (  $anonimus->{queueName} eq $queueName ) {
	    $anonimus->{queueName} = $newQueueName;
	}
    }
}

#________________________________________________________________________________
#
#  subroutine:  getQueueProperty
#   
#  description: returns a property of a queue
#
#  parameters:  queueName - the name of the Queue
#               property  - the name of the property searched
#                    available properties :
#                       "printer" - the name of the printer
#                       "XML"     - the XML file associated with the printer
#                       "PPD"     - the PPD file of the printer
#                       "type"    - the transport type
#                       "address" - the IP address of the printer
#                                   is valid only if "type" is "TCP/IP"
#                       "port"    - the port of the printer
#                                   is valid only if "type" is "Serial" or 
#                                                                    "Parallel"
#                       "language" - is the lang. of the job : Pcl , PS , Raw
#
#  return:      the 'property' value for the 'queueName' if exists
#               otherways  'undef'  value .
#________________________________________________________________________________
#
sub getQueueProperty {
    my $this = shift;
    my $queues = $this->{queues};
    my $result = undef;

    if ( @_ < 2 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter .\n");
    }
    my $searchKey = shift;
    my $property  = shift;

    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $searchKey ) {
	    $result = $anonimus->{$property};
	    last;
	}
    }
    return $result;
} 
#________________________________________________________________________________
#
#  subroutine:  setQueueProperty
#   
#  description: sets the propertyes  of a queue
#
#  parameters:  queueName - the name of the Queue
#               property  - the name of the property searched
#                    available properties :
#                       "printer" - the name of the printer
#                       "XML"     - the XML file associated with the printer
#                       "PPD"     - the PPD file of the printer
#                       "type"    - the transport type
#                       "address" - the IP address of the printer
#                                   is valid only if "type" is "TCP/IP"
#                       "port"    - the port of the printer
#                                   is valid only if "type" is "Serial" or 
#                                                                    "Parallel"
#                       "language" - is the lang. of the job : Pcl , PS , Raw
#
#               propertyValue - the value of the property
#
#  return:      ( no value ) 
#________________________________________________________________________________
#
sub setQueueProperty {
    my $this = shift;
    my $queues = $this->{queues};

    if ( @_ < 3 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter .\n");
    }
    my $searchKey = shift;
    my $property  = shift;
    my $propertyValue = shift;

    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $searchKey ) {
	    $anonimus->{$property} = $propertyValue;
	    last;
	}
    }
} 
#________________________________________________________________________________
#
#  subroutine:  getQueueOptions
#   
#  description: returns a hash with all options ( different from default ) 
#               of the printer
#               associated with a queue
#
#  parameters:  queueName - the name of the Queue
#
#  return:      the hash with all options ( different from default ) 
#               of the printer associated the 'queueName' if exists 
#               otherways  an empty  hash  .
#________________________________________________________________________________
#
sub getQueueOptions {
    my $this = shift;
    my $queues = $this->{queues};
    my %result = ();

    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter .\n");
    }
    my $searchKey = shift;

    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $searchKey ) {
	    %result = %{$anonimus->{options}}; 
	    last;
	}
    }

    return \%result;
} 
#________________________________________________________________________________
#
#  subroutine:  setQueueOptions
#   
#  description: add  all options ( different from default ) 
#               of the printer associated with a queue
#
#  parameters:  queueName   - the name of the Queue
#               hashOptions - the options
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub setQueueOptions {
    my $this = shift;
    my $queues = $this->{queues};

    if ( @_ < 2 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter .\n");
    }
    my $searchKey   = shift;
    my $hashOptions = shift;

    # erase all keys which has undef values
    my $newHash = {};
    foreach my $key ( keys %$hashOptions ) {
	if ( $hashOptions->{$key}) {
	    $newHash->{$key} = $hashOptions->{$key};
	}
    }

    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $searchKey ) {
	    $anonimus->{options}=$newHash; 
	    last;
	}
    }
} 


#________________________________________________________________________________
#
#  subroutine:  addClonedQueue
#   
#  description: add a cloned queue in order to be edited  
#   
#  parameters:  queueName - the name of the queue which is cloned
#               clonedQueueName - the name of the cloned queue
#
#  return:      ( vo value )
#________________________________________________________________________________
#
sub addClonedQueue {
    my $this = shift;
    my $queues = $this->{queues};
    if ( @_ < 2 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }
    my $queueName   = shift;
    my $clonedQueueName = shift;

    my $anonimus = {};
    $anonimus->{"queueName"}=$clonedQueueName;
    $anonimus->{"editedQueueName"}=$queueName;
    $anonimus->{"queueType"}="edit";
    @$queues = ( @$queues , $anonimus );
    if ( $queueName ) {
	$this->setQueueProperty($clonedQueueName,"printer",
				$this->getQueueProperty($queueName,"printer"));
	$this->setQueueProperty($clonedQueueName,"XML",
				$this->getQueueProperty($queueName,"XML"));
	$this->setQueueProperty($clonedQueueName,"PPD",
				$this->getQueueProperty($queueName,"PPD"));
	$this->setQueueProperty($clonedQueueName,"type",
				$this->getQueueProperty($queueName,"type"));
	$this->setQueueProperty($clonedQueueName,"address",
				$this->getQueueProperty($queueName,"address"));
	$this->setQueueProperty($clonedQueueName,"port",
				$this->getQueueProperty($queueName,"port"));
	$this->setQueueProperty($clonedQueueName,"language",
				$this->getQueueProperty($queueName,"language"));
	$this->setQueueOptions($clonedQueueName,
			       $this->getQueueOptions($queueName));
    }
    else {
	$anonimus->{options}={};
    }
}
#________________________________________________________________________________
#
#  subroutine:  IsCloned
#   
#  description: verify if the queue is a cloned queue  
#   
#  parameters:  queueName - the name of the queue
#
#  return:      1 - if the queue is a clone
#               0 - if the queue is not a clone
#________________________________________________________________________________
#
sub isCloned {
    my $this = shift;
    my $queues = $this->{queues};
    my $result = 0;
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }
    my $queueName   = shift;
    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $queueName and 
	     $anonimus->{queueType} eq "edit" ) {
	    $result = 1;
	    last;
	}
    }
    return $result;
}
#________________________________________________________________________________
#
#  subroutine:  IsQueue
#   
#  description: verify if the queue esist in the list  
#   
#  parameters:  queueName - the name of the queue
#
#  return:      1 - if the queue exists
#               0 - if the queue doesn't exist
#________________________________________________________________________________
#
sub isQueue {
    my $this = shift;
    my $queues = $this->{queues};
    my $result = 0;
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }
    my $queueName   = shift;
    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $queueName ) {
	    $result = 1;
	    last;
	}
    }
    return $result;
}
#________________________________________________________________________________
#
#  subroutine:  getClonedQueueName
#   
#  description: get the real name of the cloned queue  
#   
#  parameters:  clonedQueueName - the name of the cloned queue
#
#  return:      the real name of the cloned queue
#________________________________________________________________________________
#
sub getClonedQueueName {
    my $this   = shift;
    my $queues = $this->{queues};
    my $result = undef;
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }
    my $clonedQueueName   = shift;
    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $clonedQueueName and 
	     $anonimus->{queueType} eq "edit" ) {
	    $result = $anonimus->{editedQueueName};
	    last;
	}
    }
    return $result;
}

#________________________________________________________________________________
#
#  subroutine:  cleanQueueGarbage
#   
#  description: clean the queue informations added during edit ...  
#   
#  parameters:  queueName - the name of the  queue
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub cleanQueueGarbage  {
    my $this   = shift;
    my $queues = $this->{queues};
    my $result = undef;
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }
    my $queueName   = shift;
    foreach my $anonimus (@$queues) {
	if ( $anonimus->{queueName} eq $queueName ) {
	    $anonimus->{editedQueueName} = undef;
	    $anonimus->{type} = undef;
	    last;
	}
    }
    return $result;
}


#________________________________________________________________________________
#
#  subroutine:  writePrintcapFile
#   
#  description: update the printcap file  
#   
#  parameters:  printcapFile - the full path to printcap file
#               filterExe    - the full path of the filter  
#
#  return:      the error which ocured
#               'undef' if there is no error  
#________________________________________________________________________________
#
sub writePrintcapFile  {
    my $this   = shift;
    my $queues = $this->{queues};
    my $introduction1 = "# the following lines are inserted by IBMnpmenu";
    my $introduction2 = "# please do not edit";
    my $introduction3 = "#";
    my $endQueueIBMnpmenu = ':if=';
    if ( @_ < 3 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 3 parameter(s) .\n");
    }

    my $printcapFile   = shift;
    my $filterExe      = shift;
    my $datFolder      = shift;
    my @buffer = ();
    my @outBuffer = ();
#  open printcap file and read all data from it ...
    if( -e $printcapFile) {
	open (PRC , $printcapFile);
	@buffer = <PRC>;
	close(PRC);
    }
# clean the buffer ...
    my $semaphore = 0;
    foreach $_ (@buffer) {
	if(m/$introduction1/) {
	    $semaphore = 1;
	    next;
	}
	if( $semaphore and m/$endQueueIBMnpmenu/ ) {
	    $semaphore = 0;
	    next;
	}
	if (! $semaphore ) {
	    @outBuffer = ( @outBuffer , $_ );
	}
    }
# continue the outBuffer with queues defined by IBMnpmenu ... 

# write data in the printcap file 
    foreach my $anonimus (@$queues) {
	@outBuffer = ( @outBuffer , $introduction1                ."\n");
	@outBuffer = ( @outBuffer , $introduction2                ."\n");
	@outBuffer = ( @outBuffer , $introduction3                ."\n");
	@outBuffer = ( @outBuffer , $anonimus->{queueName}.":\\"  ."\n");
	@outBuffer = ( @outBuffer , "\t:force_localhost\@:\\"     ."\n");
	@outBuffer = ( @outBuffer , "\t:try_localhost\@:\\"       ."\n");
	@outBuffer = ( @outBuffer , "\t:mx#0:\\"                  ."\n");
	@outBuffer = ( @outBuffer , "\t:sh:\\"                    ."\n");
	@outBuffer = ( @outBuffer , "\t:sd=/var/spool/lpd/lp:\\"  ."\n");
	@outBuffer = ( @outBuffer , "\t:lpr_bounce:\\"            ."\n");
	my $printerPortAddress="";
	$printerPortAddress.=":lp=";
# if the printer is TCP/IP i have to write :lp=pppp@aaaaaaaaaaaa:\ 
#                   serial or parallel     :lp=xxxxx:\
	if ($anonimus->{type} eq "TCP/IP" ) {
	    $printerPortAddress.="PASS"."@";
	    $printerPortAddress.=$anonimus->{address};
	}
	else {
	    $printerPortAddress.=$anonimus->{port};
	}
	@outBuffer = (@outBuffer , "\t".$printerPortAddress.":");

	if( $anonimus->{language} eq  "PS" ) {
# setup the filter 
	    @outBuffer = (@outBuffer , "\\\n");
	    my $filterString="";
	    $filterString.=":if=";
	    $filterString.=$filterExe;
	    $filterString.=" --ppdfile ";
	    $filterString.=$anonimus->{PPD};
	    $filterString.=" --default ";
	    $filterString.=$datFolder."/".$anonimus->{queueName}.".def";
	    $filterString.=" --filter ";
	    $filterString.=":";
	    @outBuffer = (@outBuffer , "\t".$filterString            ."\n");
	}
    }
    open (PRC , ">".$printcapFile);
    print PRC @outBuffer;
    close(PRC);
    

}


#________________________________________________________________________________
#
#  subroutine:  writeDefFile
#   
#  description: write the .def file for the queue  
#   
#  parameters:  pathToDef
#
#________________________________________________________________________________
#
sub writeDefFiles  {

    my $this   = shift;
    my $queues = $this->{queues};
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }

    my $path   = shift;
    foreach my $anonimus (@$queues) {
	my $defFilename = $path."/".$anonimus->{queueName}.".def";
	my $options = $anonimus->{options};
	open ( DEF , ">".$defFilename);
	foreach my $key  ( keys %$options ) {
	    if ( $options->{$key} ) {
		print DEF "$key = ".$options->{$key}." \n";
	    }
	}
	close ( DEF );
    }

}


#################################################################################
#################################################################################
#
#                  package  Support::messages
#
#################################################################################
#################################################################################
package Support::messages;

use strict;
use XML::DOM;
use vars qw( @ISA );

@ISA = qw( Support::base );


#________________________________________________________________________________
#
#  package:     Support::messages
#
#  subroutine:  new
#   
#  description: the constructor of the class   
#
#  parameters:  
#               XMLfilename  - the full name  of the XML file 
#               PAFfilename  - the full name  of the PAF file 
#
#  return:      the reference to the created object 
#
#________________________________________________________________________________
#	
sub new {
    my $class = shift;


    my $this=new Support::base( @_ );

    my $trans = {};

    $this->{trans} = $trans;

    bless($this,$class);

    if ( ! $this->isValid ) {
	$this->fillFromXML;
	$this->writeToPAF;
    }
    else {
	$this->fillFromPAF;
    }
    return $this;   
}

#________________________________________________________________________________
#
#  package:     Support::messages
#
#  subroutine:  fillFromXML
# 
#  description: parse XML file and fill data members with coresponding values  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#
#________________________________________________________________________________
#		
sub fillFromXML {
    my $this        = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $trans      = {};
    $this->{trans} = $trans;

    my $languagesParser = new XML::DOM::Parser;
    my $languagesDoc    = $languagesParser->parsefile( $XMLfilename );
    my $languagesList   = $languagesDoc->getElementsByTagName("messages");
    my $currentLanguage=$languagesList->item(0);
    $this->{language} =  $currentLanguage->getAttribute("xml:lang");
    my $translationsList = $currentLanguage->getChildNodes;
    for( my $j=0; $j<$translationsList->getLength; $j++ ) {
	my $currentTranslation = $translationsList->item($j);
	my $nodeType = $currentTranslation->getNodeType;
	if ( $nodeType eq 1 ) { # is a element node ...
	    my $key   = $currentTranslation->getNodeName; 
	    my $currentText = $currentTranslation->getLastChild;
      my $value =$this->eject_escape ( $currentText->toString );
	    $trans->{$key}=$value;
	}	    
    }
}


#________________________________________________________________________________
#
#  package:     Support::messages
#
#  subroutine:  writeToPAF
#   
#  description: write data memebers to PAF file  
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub writeToPAF {
    my $this        = shift;
    my $PAFfilename = $this->{PAFfilename};
    my $trans   = $this->{trans};
    
    open (OUT_PAF , ">".$PAFfilename);
    # the first line is the control string
    print OUT_PAF ("## ",time,"\n");
    print OUT_PAF ("##\n");
    print OUT_PAF ("## this code is generated by IBMnpmenu\n");
    print OUT_PAF ("## please don\'t edit !!! \n");
    print OUT_PAF ("##\n");
    print OUT_PAF ("#language\n");
    my $language_value=$this->{language};
    print OUT_PAF ($language_value."\n");
    print OUT_PAF "#translations\n";
    foreach my $key (keys %$trans) {
	print OUT_PAF ("# ".$key."\n");
	my $value = $trans->{$key};
	print OUT_PAF ($value."\n");
    }
    print OUT_PAF ("#end_translations\n");
    print OUT_PAF ("#end_language\n");
    close (OUT_PAF);
}



#________________________________________________________________________________
#
#  package:     Support::messages
#
#  subroutine:  fillFromPAF
#   
#  description: fill data members of the class form PAF 
#
#  parameters:  ( no parameters )
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub fillFromPAF {
    my $this = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $PAFfilename = $this->{PAFfilename};
    my $trans = {}; 
    $this->{trans} = $trans;

    my $key = undef;
    my $value;
    my $language = undef;

    my @buffer;
    open ( IN_PAF , $PAFfilename );
    @buffer=<IN_PAF>;
    close( IN_PAF );

    # start to parse the buffer ...
    chop(@buffer);
    $key = undef;
    my $status = "undef";         #  this can be "undef" , "lang" or "trans" 
    foreach    my $item  (@buffer) {
        # out with blank lines 
	if ( $item eq '' ) {
	    next;
	}
        # out with comments 
	if ( substr($item,0,2) eq '##' ) {
	    next;
	}
        # start to parse queues
	if ( substr($item,0,9) eq '#language' ) {      # start a new language
            # get ready for start a new language
	    $status = "lang";
	    $key = "language";
	    next;
	}
	if ( substr($item,0,13) eq '#end_language' ) { # end the curent language
            # get ready to save the language
	    $status = "undef";
	    next;
	}

	if ( substr($item,0,13) eq '#translations' ) { 
            # get ready to fill in the translation hash
	    $status = "trans";
	    next;
	}

	if ( substr($item,0,17) eq '#end_translations' ) { 
	    $status = "lang";
	    next;
	}


       if (!  $key  ) {
           $key = substr($item,2);
           next;
        }
	else {
	    $value=$item;
	    if( $status eq "lang" ) {
		$this->{$key}=$value;
	    }
	    else {
		$trans->{$key}=$value;
	    }
	    $key = undef;
	}
 
    } 
    
}



#________________________________________________________________________________
#
#  package:     Support::messages
#
#  subroutine:  translate
#   
#  description: translate a text in the object language 
#
#  parameters:  text - the text that will be translated
#
#  return:      ( no value )
#________________________________________________________________________________
#
sub translate {
    my $this = shift;
    my $XMLfilename = $this->{XMLfilename};
    my $PAFfilename = $this->{PAFfilename};
    my $trans = $this->{trans};
    my $result = undef;

    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter .\n");
    }
    my $key = shift; 
    $result=$trans->{$key};

    if (! $result ){
	warn("Support::messages : no translation found for \'$key\'",
	     "return :  $key \n");
	$result = $key;
    }

    return $result;
}
1;














